#!/usr/bin/env python3.7
import glob, re, os, shutil
# Character set for conduit names:
# : separates the tag from the rest of the conduit name.
# Lowercase letters at end (but before :) indicate conduit variants.
# _ separates multiple output descriptors.
# * is used with symmetric-input conduits (pi, QB) to denote
# that the output object may be reflected from the form shown
# (F <-> Fx, L <-> Rx, R <-> Lx, B <-> Bx).

# Converts an output descriptor to a tuple.
nonss_re = re.compile(r"([FLRB])([x*]?)(\d+)")
dir_lookup = ("F", "L", "R", "B", "NE", "N", "NW", "W", "SW", "S", "SE", "E")
mod_lookup = ("", "x", "*")
nonss_out_lookup = ("B", "C", "D", "H", "P", "Q", "R", "W")
ss_re = re.compile(r"([NSEW]|[NS][EW])(-?\d+)T(-?\d+)")
ss_out_lookup = ("", "L", "M")
def odesc2tuple(s):
    m = nonss_re.match(s) # non-spaceship
    if m:
        orien = dir_lookup.index(m.group(1))
        omod = mod_lookup.index(m.group(2))
        ttime = int(m.group(3))
        outtype = nonss_out_lookup.index(s[m.end():])
        return (orien, omod, ttime, outtype)
    m = ss_re.match(s) # spaceship
    if m:
        direc = dir_lookup.index(m.group(1))
        ln = int(m.group(2))
        ln = 2 * abs(ln) - (ln > 0)
        toffset = int(m.group(3))
        outtype = ss_out_lookup.index(s[m.end():])
        return (direc, ln, toffset, outtype)

# Returns a tuple of non-negative integers representing the conduit name for sorting purposes.
# The input object is the first integer in the tuple.
# Output objects are represented by four consecutive integers:
# non-spaceship -> (orientation, orientation modifier, transit time, output object)
# spaceship     -> (direction, lane number*, temporal offset, output object)
# *Increasing magnitude of lane number, i.e. 0 < 1 < -1 < 2 < -2 < 3 < -3 < ...
# The corresponding blocks in the string are separated by underscores.
# A variant letter and a tag after : are optional at the end, and are converted to their respective
# ASCII values.
variant_re = re.compile(r"[a-z]*$")
in_obj_re = re.compile(r"[BCDHLMPQRW]|G(-?\d+)?")
in_obj_lookup = ("B", "C", "D", "G", "H", "L", "M", "P", "Q", "R", "W")
def conduit_key(s):
    # Split tag from name
    triple = s.partition(":")
    work, tag = triple[0], triple[2]
    # Split variant from name
    m = variant_re.search(work)
    work, variant = work[:m.start()], m.group(0)
    # Initialise output array with the input object
    m = in_obj_re.match(work)
    work, in_obj = work[m.end():], m.group(0)
    out = [in_obj_lookup.index(in_obj[0])]
    # Add output descriptors
    for odesc in work.split('_'):
        out.extend(odesc2tuple(odesc))
    out.extend(ord(c) for c in variant)
    out.extend(ord(c) for c in tag)
    return tuple(out)

# Get the filenames and their corresponding conduit names
conduit_table = {}
for path in glob.glob("*.rle"):
    with open(path) as f:
        name = f.readline().partition(' ')[2].rstrip('\n')
        in_t, out_t, num = path[:-4].split('_')
        if (in_t, out_t) not in conduit_table: conduit_table[(in_t, out_t)] = []
        conduit_table[(in_t, out_t)].append((num, name))

os.mkdir("conduits")
for ctype, conduits in conduit_table.items():
    conduits.sort(key=lambda p: conduit_key(p[1]))
    for (k, v) in enumerate(conduits):
        oldname = f"{ctype[0]}_{ctype[1]}_{v[0]}.rle"
        newname = f"conduits/{ctype[0]}_{ctype[1]}_{k}.rle"
        shutil.copyfile(oldname, newname)
        os.remove(oldname)
# At the end, move back the renamed conduit files
for f in os.listdir("conduits"):
    shutil.move("conduits/" + f, f)
os.rmdir("conduits")
